package com.flyfish.framework.compiler.support;

import com.flyfish.framework.compiler.DynamicJavaCompiler;
import org.springframework.util.StreamUtils;

import javax.tools.JavaCompiler;
import javax.tools.JavaFileManager;
import javax.tools.JavaFileObject;
import javax.tools.ToolProvider;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 简单的java编译器，单文件，无缓存版
 * 只为了偷懒，快速撸代码
 *
 * @author wangyu
 */
public class SimpleJavaCompiler implements DynamicJavaCompiler {

    // 当前的java编译器
    private final JavaCompiler compiler = ToolProvider.getSystemJavaCompiler();
    // java文件管理器
    private final JavaFileManager fileManager = compiler.getStandardFileManager(null, Locale.SIMPLIFIED_CHINESE, StandardCharsets.UTF_8);
    // 编译参数
    private final List<String> options = Arrays.asList("-d", "./bin");
    // 编译池
    private final Map<String, Class<?>> classMap = new ConcurrentHashMap<>();

    private SimpleJavaCompiler() {
    }

    public static SimpleJavaCompiler getInstance() {
        return SingletonHolder.INSTANCE;
    }

    /**
     * 编译java源码，通过字符串
     *
     * @param name   文件名R
     * @param source 源码
     * @return 结果
     */
    @Override
    public Class<?> compile(String name, String source) throws ClassNotFoundException {
        // 存在缓存，写入
        if (classMap.containsKey(name)) {
            return classMap.get(name);
        }
        // 构建java文件对象
        JavaFileObject fileObject = new StringJavaObject(name, source);
        // 设置编译环境
        JavaCompiler.CompilationTask task = compiler.getTask(null, fileManager, null,
                options, null, Collections.singletonList(fileObject));
        // 阻塞编译
        if (task.call()) {
            Class<?> clazz = Class.forName("com.flyfish.project." + name);
            classMap.put(name, clazz);
            return clazz;
        }
        throw new RuntimeException("【java编译器】尝试编译源代码出现异常！");
    }

    @Override
    public void close() throws IOException {

    }

    private static class SingletonHolder {

        private static final SimpleJavaCompiler INSTANCE = new SimpleJavaCompiler();

    }
}
