package com.flyfish.framework.service;

import com.flyfish.framework.domain.base.Qo;
import com.flyfish.framework.domain.po.User;
import com.flyfish.framework.enums.UserType;
import com.flyfish.framework.repository.UserRepository;
import com.flyfish.framework.service.impl.BaseServiceImpl;
import com.flyfish.framework.utils.Assert;
import com.flyfish.framework.utils.StrengthUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Optional;

@Service
public class UserService extends BaseServiceImpl<User> implements UserFindService {

    @Resource
    private PasswordEncoder passwordEncoder;

    /**
     * 获取用户数据
     *
     * @param username 用户
     * @return 结果
     */
    @Override
    public Optional<User> findByUsername(String username) {
        return ((UserRepository) repository).findByUsername(username);
    }

    @Override
    public User create(User entity) {
        if (null == entity.getId() && StringUtils.isNotBlank(entity.getPassword())) {
            Assert.isTrue(StrengthUtils.isValid(entity.getPassword()), "密码强度不够，至少应该包含数字、大小写字母、符号组合");
            entity.setPassword(passwordEncoder.encode(entity.getPassword()));
        }
        if (null == entity.getType()) {
            entity.setType(UserType.ADMIN);
        }
        if (null == entity.getEnable()) {
            entity.setEnable(true);
        }
        if (null == entity.getApp()) {
            entity.setApp(false);
        }
        if (null == entity.getCode()) {
            entity.setCode(entity.getUsername());
        }
        return super.create(entity);
    }

    @Override
    public User createSelective(User entity) {
        return create(entity);
    }

    @Override
    public List<User> getList(Qo<User> query) {
        return super.getList(query);
    }

}
